"""
This module contains functions for interacting with members/roles/user setup in EZOfficeInventory
"""

import os
import time
from typing import Optional

import requests

from ezoff._auth import Decorators
from ezoff._helpers import _basic_retry, _fetch_page


@Decorators.check_env_vars
def get_members(filter: Optional[dict]) -> list[dict]:
    """
    Get members from EZOfficeInventory
    Optionally filter by email, employee_identification_number, or status
    https://ezo.io/ezofficeinventory/developers/#api-retrieve-members
    """

    if filter is not None:
        if "filter" not in filter or "filter_val" not in filter:
            raise ValueError("filter must have 'filter' and 'filter_val' keys")
        if filter["filter"] not in [
            "email",
            "employee_identification_number",
            "status",
        ]:
            raise ValueError(
                "filter['filter'] must be one of 'email', 'employee_identification_number', 'status'"
            )

    url = os.environ["EZO_BASE_URL"] + "members.api"

    page = 1
    all_members = []

    while True:
        params = {"page": page, "include_custom_fields": "true"}
        if filter is not None:
            params.update(filter)

        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get members: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get members: {e}")

        data = response.json()
        if "members" not in data:
            raise Exception(f"Error, could not get members: {response.content}")

        all_members.extend(data["members"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

        # Potentially running into rate limiting issues with this endpoint
        # Sleep for a second to avoid this
        time.sleep(1)

    return all_members


@Decorators.check_env_vars
def get_filtered_members(filter: dict) -> list[dict]:
    """
    Get members via filtering.
    """

    valid_keys = [
        "filters[role][value]",
        "filters[team][value]",
        "filters[department][value]",
        "filters[login][value]",
        "filters[manager][value]",
        "filters[location][value]",
        "filters[active][value]",
        "filters[inactive][value]",
        "filters[external][value]",
        "filters[inactive_members_with_items][value]",
        "filters[inactive_members_with_pending_associations][value]",
        "filters[off_boarding_due_in][value]",
        "filters[off_boarding_overdue][value]",
        "filters[created_during][value]",
        "filters[creation_source][value]",
        "filters[last_logged_in_during][value]",
        "filters[last_sync_source][value]",
        "filters[synced_during][value]",
    ]

    # Remove any keys that are not valid
    filter = {k: v for k, v in filter.items() if k in valid_keys}

    # If no filter keys are provided, return all members
    if not filter:
        return get_members(None)

    url = os.environ["EZO_BASE_URL"] + "members/filter"

    page = 1
    all_members = []

    while True:
        params = {"page": page, "include_custom_fields": "true"}
        params.update(filter)

        try:
            response = _fetch_page(
                url,
                headers={
                    "Authorization": "Bearer " + os.environ["EZO_TOKEN"],
                    "Accept": "application/json",
                },
                params=params,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get members: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get members: {e}")

        data = response.json()

        if "data" not in data:
            raise Exception(f"Error, could not get members: {response.content}")

        all_members.extend(data["data"])

        if "total_pages" not in data["meta"]:
            break

        if page >= data["meta"]["total_pages"]:
            break

        page += 1

        # Potentially running into rate limiting issues with this endpoint
        # Sleep for a second to avoid this
        time.sleep(1)

    return all_members


@_basic_retry
@Decorators.check_env_vars
def get_member_details(member_id: int) -> dict:
    """
    Get member from EZOfficeInventory by member_id
    https://ezo.io/ezofficeinventory/developers/#api-member-details
    """

    url = os.environ["EZO_BASE_URL"] + "members/" + str(member_id) + ".api"

    try:
        response = requests.get(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            params={"include_custom_fields": "true"},
            timeout=30,
        )
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not get member details: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error getting member details: {e}")

    return response.json()


@Decorators.check_env_vars
def create_member(member: dict) -> dict:
    """
    Create a new member
    https://ezo.io/ezofficeinventory/developers/#api-create-member
    """

    # Required fields
    if "user[email]" not in member:
        raise ValueError("member must have 'user[email]' key")
    if "user[first_name]" not in member:
        raise ValueError("member must have 'user[first_name]' key")
    if "user[last_name]" not in member:
        raise ValueError("member must have 'user[last_name]' key")
    if "user[role_id]" not in member:
        raise ValueError("member must have 'user[role_id]' key")

    # Remove any keys that are not valid
    valid_keys = [
        "user[email]",
        "user[employee_id]",
        "user[employee_identification_number]",
        "user[role_id]",
        "user[team_id]",
        "user[user_listing_id]",
        "user[first_name]",
        "user[last_name]",
        "user[address_name]",
        "user[address]",
        "user[address_line_2]",
        "user[city]",
        "user[state]",
        "user[country]",
        "user[phone_number]",
        "user[fax]",
        "user[login_enabled]",
        "user[subscribed_to_emails]",
        "user[display_picture]",
        "user[unsubscribed_by_id]",
        "user[authorization_amount]",
        "user[vendor_id]",
        "user[time_zone]",
        "user[hourly_rate]",
        "user[offboarding_date]",
        "user[location_id]",
        "user[default_address_id]",
        "user[description]",
        "user[department]",
        "skip_confirmation_email",
    ]

    # Check for custom attributes
    member = {
        k: v
        for k, v in member.items()
        if k in valid_keys or k.startswith("user[custom_attributes]")
    }

    url = os.environ["EZO_BASE_URL"] + "members.api"

    try:
        response = requests.post(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=member,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not create member: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error creating member: {e}")

    return response.json()


@Decorators.check_env_vars
def update_member(member_id: int, member: dict) -> dict:
    """
    Update a member
    Note: If updating a customer that has an email, you should include the email
    in the member dict. If you don't, it will get removed for some reason. Not sure
    why as I'm using patch. So presumably should only be touching the keys that
    are specified.
    https://ezo.io/ezofficeinventory/developers/#api-update-member
    """

    # Remove any keys that are not valid
    valid_keys = [
        "user[email]",
        "user[employee_id]",
        "user[employee_identification_number]",
        "user[role_id]",
        "user[team_id]",
        "user[user_listing_id]",
        "user[first_name]",
        "user[last_name]",
        "user[address_name]",
        "user[address]",
        "user[address_line_2]",
        "user[city]",
        "user[state]",
        "user[country]",
        "user[phone_number]",
        "user[fax]",
        "user[login_enabled]",
        "user[subscribed_to_emails]",
        "user[display_picture]",
        "user[unsubscribed_by_id]",
        "user[authorization_amount]",
        "user[vendor_id]",
        "user[time_zone]",
        "user[hourly_rate]",
        "user[offboarding_date]",
        "user[location_id]",
        "user[default_address_id]",
        "user[description]",
        "user[department]",
        "skip_confirmation_email",
    ]

    # Check for custom attributes
    member = {
        k: v
        for k, v in member.items()
        if k in valid_keys or k.startswith("user[custom_attributes]")
    }

    url = os.environ["EZO_BASE_URL"] + "members/" + str(member_id) + ".api"

    try:
        response = requests.patch(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=member,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not update member: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error updating member: {e}")

    return response.json()


@Decorators.check_env_vars
def deactivate_member(member_id: int) -> dict:
    """
    Deactivate a member
    https://ezo.io/ezofficeinventory/developers/#api-deactivate-user
    """

    url = os.environ["EZO_BASE_URL"] + "members/" + str(member_id) + "/deactivate.api"

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not deactivate member: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error deactivating member: {e}")

    return response.json()


@Decorators.check_env_vars
def activate_member(member_id: int) -> dict:
    """
    Activate a member
    https://ezo.io/ezofficeinventory/developers/#api-activate-user
    """

    url = os.environ["EZO_BASE_URL"] + "members/" + str(member_id) + "/activate.api"

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not activate member: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error activating member: {e}")

    return response.json()


@Decorators.check_env_vars
def get_custom_roles() -> list[dict]:
    """
    Get list of custom roles
    Results are technically paginated but the number of custom roles
    is usually small enough that it can be returned in one page.
    https://ezo.io/ezofficeinventory/developers/#api-retrieve-roles
    """

    url = os.environ["EZO_BASE_URL"] + "custom_roles.api"

    pages = 1
    all_custom_roles = []

    while True:
        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params={"page": pages},
            )
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not update member: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error updating member: {e}")

        data = response.json()

        if "custom_roles" not in data:
            raise Exception(f"Error, could not get custom roles: {response.content}")

        all_custom_roles.extend(data["custom_roles"])

        if "total_pages" not in data:
            break

        if pages >= data["total_pages"]:
            break

        pages += 1

    return all_custom_roles


@Decorators.check_env_vars
def get_teams() -> list[dict]:
    """
    Get teams
    https://ezo.io/ezofficeinventory/developers/#api-retrieve-teams
    """

    url = os.environ["EZO_BASE_URL"] + "teams.api"

    page = 1
    all_teams = []

    while True:
        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params={"page": page},
            )
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get teams: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error getting teams: {e}")

        data = response.json()

        if "teams" not in data:
            raise Exception(f"Error, could not get teams: {response.content}")

        all_teams.extend(data["teams"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

    return all_teams



"""
Covers everything related to groups and subgroups in EZOfficeInventory
"""

import os
from typing import Optional

import requests

from ezoff._auth import Decorators
from ezoff._helpers import _basic_retry, _fetch_page


@Decorators.check_env_vars
def get_subgroups(group_id: Optional[int]) -> list[dict]:
    """
    Get subgroups
    Optionally takes a group_id to get subgroups of a specific group
    """

    url = os.environ["EZO_BASE_URL"] + "groups/get_sub_groups.api"

    params = {}

    if group_id:
        params["group_id"] = group_id

    page = 1

    all_subgroups = []

    while True:
        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get subgroups: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get subgroups: {e}")

        data = response.json()

        if "sub_groups" not in data:
            raise Exception(f"Error, could not get subgroups: {response.content}")

        all_subgroups.extend(data["sub_groups"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

    return all_subgroups



from enum import Enum


class AssetException(Exception):
    """Parent class for exceptions related to EZ-Office Fixed Assets."""

    def __init__(self, asset_id: str):
        super().__init__("")
        self.asset_id = asset_id


class AssetNotFound(AssetException):

    def __str__(self):
        return f"Asset {self.asset_id} not found in EZ-Office."


class ChecklistLinkError(Exception):
    pass


class NoDataReturned(Exception):
    pass


class WorkOrderUpdateError(Exception):
    pass



import os
import sys
from datetime import datetime, timedelta
from pprint import pprint

from dotenv import load_dotenv

# Load env vars from a .env file
load_dotenv()

sys.path.insert(0, "")

from ezoff import *

res = ezoff.get_filtered_members({"filters[location][value]": 148121})
print(res)
pass



"""
Covers everything related to fixed assets in EZOffice
"""

import os
import time

import requests

from ezoff.exceptions import *
from ezoff._auth import Decorators
from ezoff._helpers import _basic_retry, _fetch_page


@_basic_retry
@Decorators.check_env_vars
def get_asset_details(asset_id: int):
    """
    Gets asset details
    https://ezo.io/ezofficeinventory/developers/#api-asset-details
    """

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + ".api"

    try:
        response = requests.get(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data={
                "include_custom_fields": "true",
                "show_document_urls": "true",
                "show_image_urls": "true",
                "show_services_details": "true",
            },
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not get asset details: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not get asset details: {e}")

    return response.json()


@Decorators.check_env_vars
def get_all_assets() -> list[dict]:
    """
    Get assets
    Recommended to use endpoint that takes a filter instead.
    This endpoint can be slow as it returns all assets in the system. Potentially
    several hundred pages of assets.
    https://ezo.io/ezofficeinventory/developers/#api-retrive-assets
    """

    url = os.environ["EZO_BASE_URL"] + "assets.api"

    page = 1
    all_assets = []

    while True:
        params = {
            "page": page,
            "include_custom_fields": "true",
            "show_document_urls": "true",
            "show_image_urls": "true",
        }

        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
            )
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get assets: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get assets: {e}")

        data = response.json()

        if "assets" not in data:
            raise Exception(f"Error, could not get assets: {response.content}")

        all_assets.extend(data["assets"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

        # Potentially running into rate limiting issues with this endpoint
        # Sleep for a second to avoid this
        time.sleep(1)

    return all_assets


@Decorators.check_env_vars
def get_filtered_assets(filter: dict) -> list[dict]:
    """
    Get assets via filtering. Recommended to use this endpoint rather than
    returning all assets.
    """
    if "status" not in filter:
        raise ValueError("filter must have 'status' key")

    url = os.environ["EZO_BASE_URL"] + "assets/filter.api"

    page = 1
    all_assets = []

    while True:
        params = {
            "page": page,
            "include_custom_fields": "true",
            "show_document_urls": "true",
            "show_image_urls": "true",
            "show_services_details": "true",
        }
        params.update(filter)

        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
            )
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get assets: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get assets: {e}")

        data = response.json()

        if "assets" not in data:
            raise Exception(f"Error, could not get assets: {response.content}")

        all_assets.extend(data["assets"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

        # Potentially running into rate limiting issues with this endpoint
        # Sleep for a second to avoid this
        time.sleep(1)

    return all_assets


@Decorators.check_env_vars
def search_for_asset(search_term: str) -> list[dict]:
    """
    Search for an asset.
    The equivalent of the search bar in the EZOfficeInventory UI.
    May not return all assets that match the search term. Better to use
    get_filtered_assets if you want to return all assets that match a filter.
    https://ezo.io/ezofficeinventory/developers/#api-search-name

    Prefixing the search term with @ results in a search on the Asset Identification Number.
    """

    url = os.environ["EZO_BASE_URL"] + "search.api"

    page = 1
    all_assets = []

    while True:
        data = {
            "page": page,
            "search": search_term,
            "facet": "FixedAsset",
            "include_custom_fields": "true",
            "show_document_urls": "true",
            "show_image_urls": "true",
            "show_document_details": "true",
            "show_services_details": "true",
        }

        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                data=data,
            )
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get search results: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get search results: {e}")

        data = response.json()

        # Results contains multiple assets.
        if "assets" in data:
            all_assets.extend(data["assets"])

            if "total_pages" not in data:
                break

            if page >= data["total_pages"]:
                break

            page += 1

        # Results contains a single asset.
        elif "asset" in data:
            asset_list = []
            asset_list.append(data["asset"])
            return asset_list

        else:
            raise Exception(f"Error, could not get search results: {response.content}")

    return all_assets


@Decorators.check_env_vars
def create_asset(asset: dict) -> dict:
    """
    Create an asset
    https://ezo.io/ezofficeinventory/developers/#api-create-asset
    """

    # Required fields
    if "fixed_asset[name]" not in asset:
        raise ValueError("asset must have 'fixed_asset[name]' key")
    if "fixed_asset[group_id]" not in asset:
        raise ValueError("asset must have 'fixed_asset[group_id]' key")
    if "fixed_asset[purchased_on]" not in asset:
        raise ValueError("asset must have 'fixed_asset[purchased_on]' key")
        # Also check that the date is in the correct format mm/dd/yyyy
        try:
            datetime.strptime(asset["fixed_asset[purchased_on]"], "%m/%d/%Y")
        except ValueError:
            raise ValueError(
                "asset['fixed_asset[purchased_on]'] must be in the format mm/dd/yyyy"
            )

    # Remove any keys that are not valid
    valid_keys = [
        "fixed_asset[name]",
        "fixed_asset[description]",
        "fixed_asset[product_model_number]",
        "fixed_asset[manufacturer]",
        "fixed_asset[group_id]",
        "fixed_asset[sub_group_id]",
        "fixed_asset[purchased_on]",
        "fixed_asset[price]",
        "fixed_asset[location_id]",
        "fixed_asset[image_url]",
        "fixed_asset[document_urls][]",
        "fixed_asset[identifier]",
    ]

    asset = {
        k: v for k, v in asset.items() if k in valid_keys or k.startswith("cust_attr")
    }

    url = os.environ["EZO_BASE_URL"] + "assets.api"

    try:
        response = requests.post(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=asset,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not create asset: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not create asset: {e}")

    return response.json()


@Decorators.check_env_vars
def update_asset(asset_id: int, asset: dict) -> dict:
    """
    Update an asset's details
    https://ezo.io/ezofficeinventory/developers/#api-update-asset
    """

    # Remove any keys that are not valid
    valid_keys = [
        "fixed_asset[name]",
        "fixed_asset[description]",
        "fixed_asset[product_model_number]",
        "fixed_asset[manufacturer]",
        "fixed_asset[group_id]",
        "fixed_asset[sub_group_id]",
        "fixed_asset[identifier]",
        "fixed_asset[purchased_on]",
        "fixed_asset[price]",
        "fixed_asset[location_id]",
        "fixed_asset[image_url]",
        "fixed_asset[document_urls][]",
    ]

    asset = {
        k: v for k, v in asset.items() if k in valid_keys or k.startswith("cust_attr")
    }

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + ".api"

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=asset,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not update asset: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not update asset: {str(e)}")

    return response.json()


@_basic_retry
@Decorators.check_env_vars
def delete_asset(asset_id: int) -> dict:
    """
    Delete an asset
    https://ezo.io/ezofficeinventory/developers/#api-delete-asset
    """

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + ".api"

    try:
        response = requests.delete(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not delete asset: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not delete asset: {e}")

    return response.json()


@_basic_retry
@Decorators.check_env_vars
def checkin_asset(asset_id: int, checkin: dict) -> dict:
    """
    Check in an asset to a location
    https://ezo.io/ezofficeinventory/developers/#api-checkin-asset
    """

    # Required fields
    if "checkin_values[location_id]" not in checkin:
        raise ValueError("checkin must have 'checkin[location_id]' key")

    # Remove any keys that are not valid
    valid_keys = [
        "checkin_values[location_id]",
        "checkin_values[comments]",
    ]

    checkin = {
        k: v
        for k, v in checkin.items()
        if k in valid_keys or k.startswith("checkin_values[c_attr_vals]")
    }

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + "/checkin.api"

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=checkin,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not check asset in: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not check asset in: {e}")

    return response.json()


@_basic_retry
@Decorators.check_env_vars
def checkout_asset(asset_id: int, user_id: int, checkout: dict) -> dict:
    """
    Check out an asset to a member
    https://ezo.io/ezofficeinventory/developers/#api-checkout-asset

    Note: If user is inactive, checkout will return a 200 status code but the
    asset will not be checked out. Response will contain a message.
    """

    # Remove any keys that are not valid
    valid_keys = [
        "checkout_values[location_id]",
        "checkout_values[comments]",
        "till",
        "till_time",
        "checkout_values[override_conflicting_reservations]",
        "checkout_values[override_my_conflicting_reservations]",
    ]

    checkout = {
        k: v
        for k, v in checkout.items()
        if k in valid_keys or k.startswith("checkout_values[c_attr_vals]")
    }

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + "/checkout.api"

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            params={"user_id": user_id},
            data=checkout,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not check asset out: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not check asset out: {e}")

    return response.json()


@_basic_retry
@Decorators.check_env_vars
def retire_asset(asset_id: int, retire: dict) -> dict:
    """
    Retires an asset. Asset needs to be in an available state to retire.
    https://ezo.io/ezofficeinventory/developers/#api-retire-asset

    :param asset_id: The asset ID to retire
    :param retire: A dictionary containing the retirement details. Must contain the keys fixed_asset[retire_reason_id] and fixed_asset[retired_on]
    """

    # Required fields
    if "fixed_asset[retire_reason_id]" not in retire:
        raise ValueError("retire must have 'fixed_asset[retire_reason_id]' key")
    if "fixed_asset[retired_on]" not in retire:
        raise ValueError("retire must have 'fixed_asset[retired_on]' key")
        # Also check that the date is in the correct format mm/dd/yyyy
        try:
            datetime.strptime(retire["fixed_asset[retired_on]"], "%m/%d/%Y")
        except ValueError:
            raise ValueError(
                "retire['fixed_asset[retired_on]'] must be in the format mm/dd/yyyy"
            )

    # Remove any keys that are not valid
    valid_keys = [
        "fixed_asset[retire_reason_id]",
        "fixed_asset[retired_on]",
        "fixed_asset[salvage_value]",
    ]

    retire = {k: v for k, v in retire.items() if k in valid_keys}

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + "/retire.api"

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=retire,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not retire asset: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not retire asset: {e}")

    return response.json()


@_basic_retry
@Decorators.check_env_vars
def reactivate_asset(asset_id: int, reactivate: dict) -> dict:
    """
    Reactivates a retired asset.
    https://ezo.io/ezofficeinventory/developers/#api-activate-asset

    :param asset_id: The asset ID to reactivate
    :param reactivate: A dictionary containing the reactivation details. Currently that's only the key fixed_asset[location_id]. Whether it's required or not varies depending on company settings.
    """

    url = os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + "/activate.api"

    # Remove any keys that are not valid
    valid_keys = ["fixed_asset[location_id]"]

    reactivate = {k: v for k, v in reactivate.items() if k in valid_keys}

    try:
        response = requests.put(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=reactivate,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not reactivate asset: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not reactivate asset: {e}")

    return response.json()


@_basic_retry
@Decorators.check_env_vars
def verification_request(asset_id: int) -> dict:
    """
    Creates a verification request for a single asset.
    https://ezo.io/ezofficeinventory/developers/#api-verification

    Args:
        asset_id (int): The asset ID to verify.

    Raises:
        AssetNotFound: Asset ID was not found in EZ-Office.
    """

    url = (
        os.environ["EZO_BASE_URL"]
        + "assets/"
        + str(asset_id)
        + "/verification_requests.api"
    )

    try:
        response = requests.post(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            timeout=30,
        )
        response.raise_for_status()

    except (requests.exceptions.HTTPError, requests.exceptions.RequestException) as e:
        raise AssetNotFound(asset_id=str(asset_id))

    return response.json()


@Decorators.check_env_vars
def get_asset_history(asset_id: int) -> list[dict]:
    """
    Get asset history
    https://ezo.io/ezofficeinventory/developers/#api-checkin-out-history
    """

    url = (
        os.environ["EZO_BASE_URL"] + "assets/" + str(asset_id) + "/history_paginate.api"
    )

    page = 1
    all_history = []

    while True:
        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params={"page": page},
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get asset history: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error getting asset history: {e}")

        data = response.json()

        if "history" not in data:
            raise Exception(f"Error, could not get asset history: {response.content}")

        all_history.extend(data["history"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

    return all_history


@_basic_retry
@Decorators.check_env_vars
def get_items_for_token_input(q: str) -> list[dict]:
    """
    This isn't an official endpoint in the EZOfficeInventory API. It's used to populate
    the token input dropdowns in the EZOfficeInventory UI. However, still works if called
    and is needed if wanting to use the get_work_orders item filter. Which doesn't yet
    support the asset ID as a filter. But does support the ID that comes from this endpoint.
    Found this via the network tab in the browser. Not sure what the official name is
    so I'm just going off of what the URL is.

    Note: If you use "#{Asset Sequence Num}" as the q search parameter, it should
    only return one result. If you use a more general search term. like searching
    for the name, you may get multiple.
    """

    url = os.environ["EZO_BASE_URL"] + "assets/items_for_token_input.json"

    try:
        response = requests.get(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            params={"include_id": "true", "q": q},
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not get item token: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error getting item token: {e}")

    return response.json()



"""
Covers everything related to inventory assets.
"""

import os
import time

import requests

from ezoff._auth import Decorators
from ezoff._helpers import _basic_retry, _fetch_page


@Decorators.check_env_vars
def get_all_inventories() -> list[dict]:
    """
    Gets all inventory assets. Recommended to use the filtered endpoint instead
    as this one will take a long time to run.
    https://ezo.io/ezofficeinventory/developers/#api-retrieve-inventories
    """

    url = os.environ["EZO_BASE_URL"] + "inventory.api"

    page = 1
    all_inventories = []

    while True:
        params = {"page": page}

        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
                data={
                    "include_custom_fields": "true",
                    "show_document_urls": "true",
                    "show_image_urls": "true",
                    "show_document_details": "true",
                },
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get inventory details: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception("Error getting inventories: ", e)

        data = response.json()

        if "assets" not in data:
            break

        all_inventories.extend(data["assets"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

        # Potentially running into rate limiting issues with this endpoint
        # Sleep for a second to avoid this
        time.sleep(1)

    return all_inventories


# @Decorators.check_env_vars
# def get_filtered_inventories(filter: dict):
#     """
#     Gets inventory assets that match the given filters
#     https://ezo.io/ezofficeinventory/developers/#api-volatile-asset-filters
#     """

#     url = os.environ["EZO_BASE_URL"] + "inventory/filter.api"


@_basic_retry
@Decorators.check_env_vars
def get_inventory_details(inv_asset_num: int) -> dict:
    """
    Get details for an inventory asset (sometimes referred to in API as a volatile asset)
    https://ezo.io/ezofficeinventory/developers/#api-volatile-details
    """

    url = os.environ["EZO_BASE_URL"] + f"inventory/{inv_asset_num}.api"

    try:
        response = requests.get(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data={
                "include_custom_fields": "true",
                "show_document_urls": "true",
                "show_image_urls": "true",
                "show_document_details": "true",
            },
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not get inventory details: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error getting inventory details: {e}")

    return response.json()


@Decorators.check_env_vars
def create_inventory_order(inv_asset_num: int, order: dict) -> dict:
    """
    Creates an inventory order (i.e. Add Stock or New Sale)
    """
    url = os.environ["EZO_BASE_URL"] + f"inventory/{inv_asset_num}/order.api"

    # Check required fields
    if "line_item[quantity]" not in order:
        raise Exception("Quantity is required for an order")
    if "line_item[price]" not in order:
        raise Exception("Price is required for an order")
    if "order_type" not in order:
        raise Exception("Order type is required for an order")

    # Remove any keys that are not valid
    valid_keys = ["line_item[quantity]", "line_item[price]", "order_type"]

    order = {
        k: v
        for k, v in order.items()
        if k in valid_keys
        or k.startswith("add_stock_values[c_attr_vals]")
        or k.startswith("remove_stock_values[c_attr_vals]")
    }

    try:
        response = requests.post(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=order,
            timeout=30,
        )
        response.raise_for_status()
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error creating inventory order: {e}")

    return response.json()


# @Decorators.check_env_vars
# def transfer_inventory(inv_asset_num: int, transfer: dict) -> dict:
#     """
#     Transfers inventory asset amount from one location to another.
#     Called "transfer stock" in the documentation, I'm just calling it
#     transfer inventory for consistency.
#     https://ezo.io/ezofficeinventory/developers/#api-transfer-stock
#     """

#     url = os.environ["EZO_BASE_URL"] + f"inventory/{inv_asset_num}/transfer_stock.api"


@Decorators.check_env_vars
def get_inventory_history(inv_asset_num: int) -> list[dict]:
    """
    Gets history of an inventory asset
    https://ezo.io/ezofficeinventory/developers/#api-inventory-history
    """

    url = os.environ["EZO_BASE_URL"] + f"inventory/{inv_asset_num}/history.api"

    page = 1
    all_history = []

    while True:
        params = {"page": page}

        try:
            response = requests.get(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
                timeout=30,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get inventory history: {e.response.status_code} - {e.response.content}"
            )
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error getting inventory history: {e}")

        data = response.json()

        if "history" not in data:
            break

        all_history.extend(data["history"])

        if "total_pages" not in data:
            break

        if page >= data["total_pages"]:
            break

        page += 1

        # Potentially running into rate limiting issues with this endpoint
        # Sleep for a second to avoid this
        time.sleep(1)

    return all_history


# TODO Create Inventory Asset
# TODO Update Inventory Asset
# TODO Delete Inventory Asset
# TODO Get Location Based Thresholds



"""
Projects in EZOffice
"""

import os
import time

import requests
from pprint import pprint

from ezoff._auth import Decorators
from ezoff._helpers import _basic_retry, _fetch_page



@Decorators.check_env_vars
def project_link_asset(options: dict) -> dict:
    """
    Link a Fixed Asset to a Project
    https://ezo.io/ezofficeinventory/developers/#api-link-unlink-project
    """

    # Required fields
    if "seqs" not in options:
        raise ValueError("asset must have 'fixed_asset[name]' key")
    
    if "project_id" not in options:
        raise ValueError("asset must have 'fixed_asset[group_id]' key")

    # Remove any keys that are not valid
    valid_keys = [
        "seqs",
        "project_id",
    ]

    options = {
        k: v for k, v in options.items() if k in valid_keys or k.startswith("cust_attr")
    }

    url = os.environ["EZO_BASE_URL"] + "assets/link_to_project.api"

    try:
        response = requests.post(
            url,
            headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
            data=options,
            timeout=30,
        )
        response.raise_for_status()

    except requests.exceptions.HTTPError as e:
        raise Exception(
            f"Error, could not assign asset to project: {e.response.status_code} - {e.response.content}"
        )
    except requests.exceptions.RequestException as e:
        raise Exception(f"Error, could not assign asset to project: {e}")

    return response.json()



"""
Module for interacting with the EZOffice Inventory API
"""

from .assets import *
from .exceptions import *
from .groups import *
from .inventory import *
from .locations import *
from .members import *
from .projects import *
from .workorders import *



from setuptools import find_packages, setup

with open("requirements.txt") as f:
    required = f.read().splitlines()

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="ezoff",
    version="0.1.24",
    description="Python package that acts as a wrapper for the EZOffice API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Jordan Maynor",
    author_email="jmaynor@pepsimidamerica.com",
    url="https://github.com/pepsimidamerica/ezoff",
    packages=find_packages(),
    python_requires=">=3.10",
    install_requires=required,
)
